from stackboost.models.tree import SimilarityTreeRegressor, DispersionTreeRegressor, ErrorTreeRegressor, StackedTreeClassifier
from stackboost.models.gradient_boosting import StackedGradientBoostingRegressor
from stackboost.loss_functions import MSE, SquareLoss, CrossEntropy

__author__ = 'Lev Novitskiy > levnovitskiy@gmail.com > https://github.com/leffff/ > https://www.kaggle.com/levnovitskiy'
__license__ = 'MIT'
